// Copyright (c) 2014, The shelf_oauth authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_oauth_memcache.tokenstore.oauth1.impl;


import 'dart:async';
import 'package:option/option.dart';
import 'package:shelf_oauth/shelf_oauth.dart';
import 'package:memcache/memcache.dart';
import 'preconditions.dart';
import 'oauth1_token_store.dart';

class MemcacheOAuth1RequestTokenSecretStore
    implements OAuth1RequestTokenSecretStore {
  final MemcacheProvider memcacheProvider;
  final Duration expiration;

  Memcache get memcache {
    final m = memcacheProvider();
    ensure(m, isNotNull);
    return m;
  }

  MemcacheOAuth1RequestTokenSecretStore(this.memcacheProvider, this.expiration) {
    ensure(memcacheProvider, isNotNull);
    ensure(expiration, isNotNull);
  }


  @override
  Future storeSecret(String authToken, String secret) {
    return memcache.set(authToken, secret, expiration: expiration);
  }

  @override
  Future<Option<String>> consumeSecret(String authToken) {
    final result = memcache.get(authToken).then((secret) => new Option(secret));
    memcache.remove(authToken); // remove it async. Don't care the outcome
    return result;
  }

}